@@ -40,10 +40,31 @@ vcard_from_couch_document (CouchDBDocument *document)
 	char *str;
 	GSList *list, *sl;
 	GList *attr_list;
+	CouchDBStructField *app_annotations;
 
 	if (!couchdb_document_is_contact (document))
 		return NULL;
 
+	/* Check if the contact is marked for deletion */
+	if ((app_annotations = couchdb_document_get_application_annotations (document))) {
+		CouchDBStructField *u1_annotations;
+
+		u1_annotations = couchdb_struct_field_get_struct_field (
+			app_annotations, "Ubuntu One");
+		if (u1_annotations != NULL) {
+			CouchDBStructField *private_annotations;
+
+			private_annotations = couchdb_struct_field_get_struct_field (
+				u1_annotations, "private_application_annotations");
+			if (private_annotations != NULL) {
+				if (couchdb_struct_field_has_field (private_annotations, "deleted")
+				    && couchdb_struct_field_get_boolean_field (private_annotations, "deleted"))
+					return NULL;
+			}
+		}
+	}
+
+	/* Fill in the EContact with the data from the CouchDBDocument */
 	contact = e_contact_new ();
 	e_vcard_add_attribute_with_value (E_VCARD (contact),
 					  e_vcard_attribute_new (NULL, COUCHDB_REVISION_PROP),
@@ -239,13 +260,23 @@ vcard_from_couch_document (CouchDBDocument *document)
 	/* parse postal addresses */
 	list = couchdb_document_contact_get_addresses (document);
 	while (list != NULL) {
+		char **street_lines;
 		const char *description_str;
 		EContactAddress *contact_address;
 		CouchDBStructField *address = (CouchDBStructField *) list->data;
 
 		contact_address = g_new0 (EContactAddress, 1);
 		contact_address->address_format = g_strdup ("");
-		contact_address->street = g_strdup (couchdb_document_contact_address_get_street (address));
+
+		street_lines = g_strsplit (couchdb_document_contact_address_get_street (address), "\n", 2);
+		if (street_lines != NULL) {
+			contact_address->street = g_strdup (street_lines[0]);
+			if (street_lines[1] != NULL)
+				contact_address->ext = g_strdup (street_lines[1]);
+			g_strfreev (street_lines);
+		} else
+			contact_address->street = g_strdup (couchdb_document_contact_address_get_street (address));
+
 		contact_address->locality = g_strdup (couchdb_document_contact_address_get_city (address));
 		contact_address->region = g_strdup (couchdb_document_contact_address_get_state (address));
 		contact_address->country = g_strdup (couchdb_document_contact_address_get_country (address));
@@ -311,6 +342,11 @@ vcard_from_couch_document (CouchDBDocument *document)
 		address_str = couchdb_document_contact_im_get_address (im);
 		description_str = couchdb_document_contact_im_get_description (im);
 		protocol_str = couchdb_document_contact_im_get_protocol (im);
+		/* Some records don't have the 'protocol' field, and use the
+		   'description' field to specify the kind of IM address this
+		   refers to */
+		if (protocol_str == NULL)
+			protocol_str = description_str;
 		uuid_str = couchdb_struct_field_get_uuid (im);
 
 		if (protocol_str != NULL) {
@@ -411,7 +447,7 @@ contact_email_to_struct_field (EVCardAttribute *attr)
 {
 	const gchar *email;
 	GList *params, *pl;
-	const gchar *description = NULL, *uuid = NULL;
+	const gchar *description = "other", *uuid = NULL;
 
 	email = e_vcard_attribute_get_value (attr);
 	if (email == NULL)
@@ -517,6 +553,37 @@ contact_phone_to_struct_field (EVCardAttribute *attr)
 	return sf;
 }
 
+static CouchDBStructField *
+contact_postal_address_to_struct_field (EContact *contact, EContactField field, const char *description)
+{
+	CouchDBStructField *sf = NULL;
+	EContactAddress *contact_address;
+
+	contact_address = e_contact_get (contact, field);
+	if (contact_address) {
+		char *street = NULL;
+
+		if (contact_address->ext && *contact_address->ext)
+			street = g_strdup_printf ("%s\n%s", contact_address->street, contact_address->ext);
+		else if (contact_address->street && *contact_address->street)
+			street = g_strdup (contact_address->street);
+
+		sf = couchdb_document_contact_address_new (/* FIXME */ NULL,
+							   street,
+							   contact_address->locality,
+							   contact_address->region,
+							   contact_address->country,
+							   contact_address->code,
+							   contact_address->po,
+							   description);
+		
+		e_contact_address_free (contact_address);
+		g_free (street);
+	}
+
+	return sf;
+}
+
 static CouchDBStructField *
 contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 {
@@ -524,7 +591,7 @@ contact_url_to_struct_field (EVCardAttribute *attr, const gchar *description)
 	GList *params, *pl;
 
 	address = e_vcard_attribute_get_value (attr);
-	if (!address)
+	if (address == NULL || strlen (address) <= 0)
 		return NULL;
 
 	params = e_vcard_attribute_get_params (attr);
@@ -592,7 +659,7 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	const char *str;
 	CouchDBDocument *document;
 	gint i;
-	const EContactAddress *contact_address;
+	CouchDBStructField *postal_address;
 
 	/* create the CouchDBDocument to put on the database */
 	document = couchdb_document_contact_new (couchdb_backend->couchdb);
@@ -661,50 +728,17 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 
 	/* postal addresses */
 	list = NULL;
-	contact_address = e_contact_get_const (contact, E_CONTACT_ADDRESS_HOME);
-	if (contact_address) {
-		CouchDBStructField *sf;
+	postal_address = contact_postal_address_to_struct_field (contact, E_CONTACT_ADDRESS_HOME, "home");
+	if (postal_address)
+		list = g_slist_append (list, postal_address);
 
-		sf = couchdb_document_contact_address_new (/* FIXME */ NULL,
-							   contact_address->street,
-							   contact_address->locality,
-							   contact_address->region,
-							   contact_address->country,
-							   contact_address->code,
-							   contact_address->po,
-							   "home");
-		list = g_slist_append (list, sf);
-	}
+	postal_address = contact_postal_address_to_struct_field (contact, E_CONTACT_ADDRESS_WORK, "work");
+	if (postal_address)
+		list = g_slist_append (list, postal_address);
 
-	contact_address = e_contact_get_const (contact, E_CONTACT_ADDRESS_WORK);
-	if (contact_address) {
-		CouchDBStructField *sf;
-
-		sf = couchdb_document_contact_address_new (/* FIXME */ NULL,
-							   contact_address->street,
-							   contact_address->locality,
-							   contact_address->region,
-							   contact_address->country,
-							   contact_address->code,
-							   contact_address->po,
-							   "work");
-		list = g_slist_append (list, sf);
-	}
-
-	contact_address = e_contact_get_const (contact, E_CONTACT_ADDRESS_OTHER);
-	if (contact_address) {
-		CouchDBStructField *sf;
-
-		sf = couchdb_document_contact_address_new (/* FIXME */ NULL,
-							   contact_address->street,
-							   contact_address->locality,
-							   contact_address->region,
-							   contact_address->country,
-							   contact_address->code,
-							   contact_address->po,
-							   "other");
-		list = g_slist_append (list, sf);
-	}
+	postal_address = contact_postal_address_to_struct_field (contact, E_CONTACT_ADDRESS_OTHER, "other");
+	if (postal_address)
+		list = g_slist_append (list, postal_address);
 
 	if (list) {
 		couchdb_document_contact_set_addresses (document, list);
@@ -782,21 +816,25 @@ couch_document_from_contact (EBookBackendCouchDB *couchdb_backend, EContact *con
 	}
 
 	/* birth date */
-	dt = (EContactDate *) e_contact_get_const (contact, E_CONTACT_BIRTH_DATE);
+	dt = (EContactDate *) e_contact_get (contact, E_CONTACT_BIRTH_DATE);
 	if (dt) {
 		char *dt_str = e_contact_date_to_string (dt);
-		couchdb_document_contact_set_birth_date (document, (const char *) dt_str);
+		if (dt_str != NULL) {
+			couchdb_document_contact_set_birth_date (document, (const char *) dt_str);
 
-		g_free (dt_str);
+			g_free (dt_str);
+		}
 	}
 
 	/* wedding date */
-	dt = (EContactDate *) e_contact_get_const (contact, E_CONTACT_ANNIVERSARY);
+	dt = (EContactDate *) e_contact_get (contact, E_CONTACT_ANNIVERSARY);
 	if (dt) {
 		char *dt_str = e_contact_date_to_string (dt);
-		couchdb_document_contact_set_wedding_date (document, (const char *) dt_str);
+		if (dt_str != NULL) {
+			couchdb_document_contact_set_wedding_date (document, (const char *) dt_str);
 
-		g_free (dt_str);
+			g_free (dt_str);
+		}
 	}
 
 	/* application annotations */
@@ -870,6 +908,7 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 
 	/* create CouchDB main object */
 	couchdb_backend->dbname = g_strdup ("contacts");
+	couchdb_backend->using_desktopcouch = FALSE;
 
 	property = e_source_get_property (source, "couchdb_instance");
 	if (g_strcmp0 (property, "user") == 0) {
@@ -944,6 +983,8 @@ e_book_backend_couchdb_load_source (EBookBackend *backend,
 			return GNOME_Evolution_Addressbook_NoSuchBook;
 		}
 
+		couchdb_backend->using_desktopcouch = TRUE;
+
 	} else if (g_strcmp0 (property, "remote") == 0)
 		uri = g_strdup_printf ("http://%s", e_source_get_property (source, "couchdb_remote_server"));
 	else
@@ -1024,7 +1065,7 @@ e_book_backend_couchdb_remove (EBookBackend *backend, EDataBook *book, guint32 o
 static char *
 e_book_backend_couchdb_get_static_capabilities (EBookBackend *backend)
 {
-	return g_strdup ("local,do-initial-query,bulk-removes,contact-lists");
+	return g_strdup ("local,do-initial-query,bulk-removes");
 }
 
 static EContact *
@@ -1104,14 +1145,53 @@ e_book_backend_couchdb_remove_contacts (EBookBackend *backend,
 
 		document = couchdb_document_get (couchdb_backend->couchdb, couchdb_backend->dbname, uid, &error);
 		if (document) {
-			if (couchdb_document_delete (document, &error))
-				deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
-			else {
-				if (error != NULL) {
-					g_debug ("Error deleting document: %s", error->message);
-					g_error_free (error);
-				} else
-					g_debug ("Error deleting document");
+			if (couchdb_backend->using_desktopcouch) {
+				CouchDBStructField *app_annotations, *u1_annotations, *private_annotations;
+
+				/* For desktopcouch, we don't remove contacts, we just
+				 * mark them as deleted */
+				app_annotations = couchdb_document_get_application_annotations (document);
+				if (app_annotations == NULL)
+					app_annotations = couchdb_struct_field_new ();
+
+				u1_annotations = couchdb_struct_field_get_struct_field (app_annotations, "Ubuntu One");
+				if (u1_annotations == NULL)
+					u1_annotations = couchdb_struct_field_new ();
+
+				private_annotations = couchdb_struct_field_get_struct_field (u1_annotations, "private_application_annotations");
+				if (private_annotations == NULL)
+					private_annotations = couchdb_struct_field_new ();
+
+				couchdb_struct_field_set_boolean_field (private_annotations, "deleted", TRUE);
+				couchdb_struct_field_set_struct_field (u1_annotations, "private_application_annotations", private_annotations);
+				couchdb_struct_field_set_struct_field (app_annotations, "Ubuntu One", u1_annotations);
+				couchdb_document_set_application_annotations (document, app_annotations);
+
+				/* Now put the new revision of the document */
+				if (couchdb_document_put (document, couchdb_backend->dbname, &error))
+					deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
+				else {
+					if (error != NULL) {
+						g_debug ("Error deleting document: %s", error->message);
+						g_error_free (error);
+					} else
+						g_debug ("Error deleting document");
+				}
+
+				/* Free memory */
+				couchdb_struct_field_unref (app_annotations);
+				couchdb_struct_field_unref (u1_annotations);
+				couchdb_struct_field_unref (private_annotations);
+			} else {
+				if (couchdb_document_delete (document, &error))
+					deleted_ids = g_list_append (deleted_ids, (gpointer) uid);
+				else {
+					if (error != NULL) {
+						g_debug ("Error deleting document: %s", error->message);
+						g_error_free (error);
+					} else
+						g_debug ("Error deleting document");
+				}
 			}
 		} else {
 			if (error != NULL) {
@@ -1212,6 +1292,7 @@ e_book_backend_couchdb_get_contact_list (EBookBackend *backend,
 					   couchdb_backend->dbname,
 					   &error);
 	for (sl = doc_list; sl != NULL; sl = sl->next) {
+		char *vcard;
 		CouchDBDocument *document;
 		CouchDBDocumentInfo *doc_info = (CouchDBDocumentInfo *) sl->data;
 
@@ -1224,7 +1305,10 @@ e_book_backend_couchdb_get_contact_list (EBookBackend *backend,
 		if (!document)
 			continue;
 
-		contacts = g_list_prepend (contacts, vcard_from_couch_document (document));
+		vcard = vcard_from_couch_document (document);
+		if (vcard != NULL)
+			contacts = g_list_prepend (contacts, vcard);
+
 		g_object_unref (G_OBJECT (document));
 	}
 
